@@ -8,13 +8,18 @@
 
 '''Overridden Mercurial commands and functions for the largefiles extension'''
 
+from __future__ import annotations
+
 import contextlib
 import copy
 import os
+from typing import (
+    Optional,
+)
 
 from mercurial.i18n import _
 
-from mercurial.pycompat import open
+from mercurial.interfaces.types import MatcherT
 
 from mercurial.hgweb import webcommands
 
@@ -46,6 +51,8 @@ from mercurial.upgrade_utils import (
     actions as upgrade_actions,
 )
 
+from mercurial.utils import urlutil
+
 from . import (
     lfcommands,
     lfutil,
@@ -71,6 +78,7 @@ def composelargefilematcher(match, manifest):
     """create a matcher that matches only the largefiles in the original
     matcher"""
     m = copy.copy(match)
+    m._was_tampered_with = True
     lfile = lambda f: lfutil.standin(f) in manifest
     m._files = [lf for lf in m._files if lfile(lf)]
     m._fileset = set(m._files)
@@ -86,6 +94,7 @@ def composenormalfilematcher(match, manifest, exclude=None):
         excluded.update(exclude)
 
     m = copy.copy(match)
+    m._was_tampered_with = True
     notlfile = lambda f: not (
         lfutil.isstandin(f) or lfutil.standin(f) in manifest or f in excluded
     )
@@ -175,10 +184,10 @@ def removelargefiles(ui, repo, isaddremove, matcher, uipathfn, dryrun, **opts):
     with lfstatus(repo):
         s = repo.status(match=m, clean=not isaddremove)
     manifest = repo[None].manifest()
-    modified, added, deleted, clean = [
+    modified, added, deleted, clean = (
         [f for f in list if lfutil.standin(f) in manifest]
         for list in (s.modified, s.added, s.deleted, s.clean)
-    ]
+    )
 
     def warn(files, msg):
         for f in files:
@@ -442,6 +451,8 @@ def overridelog(orig, ui, repo, *pats, **opts):
 
         pats.update(fixpats(f, tostandin) for f in p)
 
+        m._was_tampered_with = True
+
         for i in range(0, len(m._files)):
             # Don't add '.hglf' to m.files, since that is already covered by '.'
             if m._files[i] == b'.':
@@ -821,11 +832,11 @@ def overridecopy(orig, ui, repo, pats, opts, rename=False):
         if not os.path.isdir(makestandin(dest)):
             os.makedirs(makestandin(dest))
 
-    try:
-        # When we call orig below it creates the standins but we don't add
-        # them to the dir state until later so lock during that time.
-        wlock = repo.wlock()
+    # When we call orig below it creates the standins but we don't add
+    # them to the dir state until later so lock during that time.
+    wlock = repo.wlock()
 
+    try:
         manifest = repo[None].manifest()
 
         def overridematch(
@@ -849,6 +860,7 @@ def overridecopy(orig, ui, repo, pats, opts, rename=False):
                     newpats.append(pat)
             match = orig(ctx, newpats, opts, globbed, default, badfn=badfn)
             m = copy.copy(match)
+            m._was_tampered_with = True
             lfile = lambda f: lfutil.standin(f) in manifest
             m._files = [lfutil.standin(f) for f in m._files if lfile(f)]
             m._fileset = set(m._files)
@@ -881,7 +893,7 @@ def overridecopy(orig, ui, repo, pats, opts, rename=False):
             ):
                 destlfile = dest.replace(lfutil.shortname, b'')
                 if not opts[b'force'] and os.path.exists(destlfile):
-                    raise IOError(
+                    raise OSError(
                         b'', _(b'destination largefile already exists')
                     )
             copiedfiles.append((src, dest))
@@ -892,7 +904,7 @@ def overridecopy(orig, ui, repo, pats, opts, rename=False):
                 result += orig(ui, repo, listpats, opts, rename)
 
         lfdirstate = lfutil.openlfdirstate(ui, repo)
-        for (src, dest) in copiedfiles:
+        for src, dest in copiedfiles:
             if lfutil.shortname in src and dest.startswith(
                 repo.wjoin(lfutil.shortname)
             ):
@@ -967,6 +979,7 @@ def overriderevert(orig, ui, repo, ctx, *pats, **opts):
                 opts = {}
             match = orig(mctx, pats, opts, globbed, default, badfn=badfn)
             m = copy.copy(match)
+            m._was_tampered_with = True
 
             # revert supports recursing into subrepos, and though largefiles
             # currently doesn't work correctly in that case, this match is
@@ -1130,7 +1143,10 @@ def overrideclone(orig, ui, source, dest=None, **opts):
     d = dest
     if d is None:
         d = hg.defaultdest(source)
-    if opts.get('all_largefiles') and not hg.islocal(d):
+    if opts.get('all_largefiles') and urlutil.url(d).scheme not in (
+        b'file',
+        None,
+    ):
         raise error.Abort(
             _(b'--all-largefiles is incompatible with non-local destination %s')
             % d
@@ -1218,7 +1234,7 @@ def overridearchive(
     node,
     kind,
     decode=True,
-    match=None,
+    match: Optional[MatcherT] = None,
     prefix=b'',
     mtime=None,
     subrepos=None,
@@ -1238,32 +1254,56 @@ def overridearchive(
     if kind not in archival.archivers:
         raise error.Abort(_(b"unknown archive type '%s'") % kind)
 
-    ctx = repo[node]
-
     if kind == b'files':
         if prefix:
             raise error.Abort(_(b'cannot give prefix when archiving to files'))
     else:
         prefix = archival.tidyprefix(dest, kind, prefix)
 
+    if not match:
+        match = scmutil.matchall(repo)
+    archiver = None
+    ctx = repo[node]
+
+    def opencallback():
+        """Return the archiver instance, creating it if necessary.
+
+        This function is called when the first actual entry is created.
+        It may be called multiple times from different layers.
+        When serving the archive via hgweb, no errors should happen after
+        this point.
+        """
+        nonlocal archiver
+        if archiver is None:
+            if callable(dest):
+                output = dest()
+            else:
+                output = dest
+            archiver = archival.archivers[kind](output, mtime or ctx.date()[0])
+            assert archiver is not None
+
+            if repo.ui.configbool(b"ui", b"archivemeta"):
+                metaname = b'.hg_archival.txt'
+                if match(metaname):
+                    write(
+                        metaname,
+                        0o644,
+                        False,
+                        lambda: archival.buildmetadata(ctx),
+                    )
+        return archiver
+
     def write(name, mode, islink, getdata):
-        if match and not match(name):
+        if not match(name):
             return
         data = getdata()
         if decode:
             data = repo.wwritedata(name, data)
+        if archiver is None:
+            opencallback()
+        assert archiver is not None, "archive should be opened by now"
         archiver.addfile(prefix + name, mode, islink, data)
 
-    archiver = archival.archivers[kind](dest, mtime or ctx.date()[0])
-
-    if repo.ui.configbool(b"ui", b"archivemeta"):
-        write(
-            b'.hg_archival.txt',
-            0o644,
-            False,
-            lambda: archival.buildmetadata(ctx),
-        )
-
     for f in ctx:
         ff = ctx.flags(f)
         getdata = ctx[f].data
@@ -1298,20 +1338,21 @@ def overridearchive(
             # allow only hgsubrepos to set this, instead of the current scheme
             # where the parent sets this for the child.
             with (
-                hasattr(sub, '_repo')
-                and lfstatus(sub._repo)
-                or util.nullcontextmanager()
+                lfstatus(sub._repo)
+                if hasattr(sub, '_repo')
+                else util.nullcontextmanager()
             ):
-                sub.archive(archiver, subprefix, submatch)
+                sub.archive(opencallback, subprefix, submatch)
 
-    archiver.done()
+    if archiver:
+        archiver.done()
 
 
 @eh.wrapfunction(subrepo.hgsubrepo, 'archive')
-def hgsubrepoarchive(orig, repo, archiver, prefix, match=None, decode=True):
+def hgsubrepoarchive(orig, repo, opener, prefix, match: MatcherT, decode=True):
     lfenabled = hasattr(repo._repo, '_largefilesenabled')
     if not lfenabled or not repo._repo.lfstatus:
-        return orig(repo, archiver, prefix, match, decode)
+        return orig(repo, opener, prefix, match, decode)
 
     repo._get(repo._state + (b'hg',))
     rev = repo._state[1]
@@ -1320,15 +1361,20 @@ def hgsubrepoarchive(orig, repo, archiver, prefix, match=None, decode=True):
     if ctx.node() is not None:
         lfcommands.cachelfiles(repo.ui, repo._repo, ctx.node())
 
+    archiver = None
+
     def write(name, mode, islink, getdata):
+        nonlocal archiver
         # At this point, the standin has been replaced with the largefile name,
         # so the normal matcher works here without the lfutil variants.
-        if match and not match(f):
+        if not match(f):
             return
         data = getdata()
         if decode:
             data = repo._repo.wwritedata(name, data)
 
+        if archiver is None:
+            archiver = opener()
         archiver.addfile(prefix + name, mode, islink, data)
 
     for f in ctx:
@@ -1364,11 +1410,11 @@ def hgsubrepoarchive(orig, repo, archiver, prefix, match=None, decode=True):
         # would allow only hgsubrepos to set this, instead of the current scheme
         # where the parent sets this for the child.
         with (
-            hasattr(sub, '_repo')
-            and lfstatus(sub._repo)
-            or util.nullcontextmanager()
+            lfstatus(sub._repo)
+            if hasattr(sub, '_repo')
+            else util.nullcontextmanager()
         ):
-            sub.archive(archiver, subprefix, submatch, decode)
+            sub.archive(opener, subprefix, submatch, decode)
 
 
 # If a largefile is modified, the change is not reflected in its
@@ -1595,6 +1641,7 @@ def scmutiladdremove(
     # confused state later.
     if s.deleted:
         m = copy.copy(matcher)
+        m._was_tampered_with = True
 
         # The m._files and m._map attributes are not changed to the deleted list
         # because that affects the m.exact() test, which in turn governs whether
@@ -1610,7 +1657,7 @@ def scmutiladdremove(
             m,
             uipathfn,
             opts.get(b'dry_run'),
-            **pycompat.strkwargs(opts)
+            **pycompat.strkwargs(opts),
         )
     # Call into the normal add code, and any files that *should* be added as
     # largefiles will be
@@ -1721,6 +1768,7 @@ def overridecat(orig, ui, repo, file1, *pats, **opts):
     err = 1
     notbad = set()
     m = scmutil.match(ctx, (file1,) + pats, pycompat.byteskwargs(opts))
+    m._was_tampered_with = True
     origmatchfn = m.matchfn
 
     def lfmatchfn(f):
@@ -1779,7 +1827,7 @@ def overridecat(orig, ui, repo, file1, *pats, **opts):
                             % lf
                         )
                 path = lfutil.usercachepath(repo.ui, hash)
-                with open(path, b"rb") as fpin:
+                with open(path, "rb") as fpin:
                     for chunk in util.filechunkiter(fpin):
                         fp.write(chunk)
         err = 0
