@@ -5,8 +5,11 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
+
 import collections
 import time
+import typing
 
 from mercurial.node import bin, hex, nullrev
 from mercurial import (
@@ -18,6 +21,11 @@ from mercurial import (
 )
 from . import shallowutil
 
+if typing.TYPE_CHECKING:
+    from typing import (
+        Iterator,
+    )
+
 propertycache = util.propertycache
 FASTLOG_TIMEOUT_IN_SECS = 0.5
 
@@ -37,9 +45,7 @@ class remotefilectx(context.filectx):
             fileid = repo.nullid
         if fileid and len(fileid) == 40:
             fileid = bin(fileid)
-        super(remotefilectx, self).__init__(
-            repo, path, changeid, fileid, filelog, changectx
-        )
+        super().__init__(repo, path, changeid, fileid, filelog, changectx)
         self._ancestormap = ancestormap
 
     def size(self):
@@ -100,7 +106,11 @@ class remotefilectx(context.filectx):
             if path in data[3]:  # checking the 'files' field.
                 # The file has been touched, check if the hash is what we're
                 # looking for.
-                if fileid == mfl[data[0]].readfast().get(path):
+                #
+                # The change has to be against a parent, otherwise we might be
+                # missing linkrev worthy changes.
+                m = mfl[data[0]].read_delta_parents(exact=False)
+                if fileid == m.get(path):
                     return rev
 
         # Couldn't find the linkrev. This should generally not happen, and will
@@ -199,8 +209,10 @@ class remotefilectx(context.filectx):
         manifestnode, files = ancctx[0], ancctx[3]
         # If the file was touched in this ancestor, and the content is similar
         # to the one we are searching for.
-        if path in files and fnode == mfl[manifestnode].readfast().get(path):
-            return cl.node(ancrev)
+        if path in files:
+            m = mfl[manifestnode].read_delta_parents(exact=False)
+            if fnode == m.get(path):
+                return cl.node(ancrev)
         return None
 
     def _adjustlinknode(self, path, filelog, fnode, srcrev, inclusive=False):
@@ -345,7 +357,7 @@ class remotefilectx(context.filectx):
                 b'linkrevfixup',
                 logmsg + b'\n',
                 elapsed=elapsed * 1000,
-                **commonlogkwargs
+                **commonlogkwargs,
             )
 
     def _verifylinknode(self, revs, linknode):
@@ -373,7 +385,7 @@ class remotefilectx(context.filectx):
             # the correct linknode.
             return False
 
-    def ancestors(self, followfirst=False):
+    def ancestors(self, followfirst=False) -> Iterator[remotefilectx]:
         ancestors = []
         queue = collections.deque((self,))
         seen = set()
@@ -399,8 +411,7 @@ class remotefilectx(context.filectx):
         # The copy tracing algorithm depends on these coming out in order
         ancestors = sorted(ancestors, reverse=True, key=lambda x: x.linkrev())
 
-        for ancestor in ancestors:
-            yield ancestor
+        yield from ancestors
 
     def ancestor(self, fc2, actx):
         # the easy case: no (relevant) renames
@@ -477,7 +488,7 @@ class remotefilectx(context.filectx):
         )
         if fetch:
             self._repo.fileservice.prefetch(fetch)
-        return super(remotefilectx, self).annotate(*args, **kwargs)
+        return super().annotate(*args, **kwargs)
 
     # Return empty set so that the hg serve and thg don't stack trace
     def children(self):
@@ -487,9 +498,7 @@ class remotefilectx(context.filectx):
 class remoteworkingfilectx(context.workingfilectx, remotefilectx):
     def __init__(self, repo, path, filelog=None, workingctx=None):
         self._ancestormap = None
-        super(remoteworkingfilectx, self).__init__(
-            repo, path, filelog, workingctx
-        )
+        super().__init__(repo, path, filelog, workingctx)
 
     def parents(self):
         return remotefilectx.parents(self)
