@@ -39,6 +39,7 @@ Config
   skiphash = False
 """
 
+from __future__ import annotations
 
 import inspect
 import os
@@ -48,6 +49,10 @@ import stat
 import struct
 import time
 
+from typing import (
+    Optional,
+)
+
 from .i18n import _
 from .node import hex
 
@@ -147,7 +152,7 @@ def _getmtimepaths(ui):
     """
     modules = [m for n, m in extensions.extensions(ui)]
     try:
-        from . import __version__
+        from . import __version__  # pytype: disable=import-error
 
         modules.append(__version__)
     except ImportError:
@@ -218,7 +223,7 @@ class hashstate:
 def _newchgui(srcui, csystem, attachio):
     class chgui(srcui.__class__):
         def __init__(self, src=None):
-            super(chgui, self).__init__(src)
+            super().__init__(src)
             if src:
                 self._csystem = getattr(src, '_csystem', csystem)
             else:
@@ -263,6 +268,7 @@ def _loadnewui(srcui, args, cdebug):
 
     # command line args
     options = dispatch._earlyparseopts(newui, args)
+    dispatch._parse_config_files(newui, args, options[b'config_file'])
     dispatch._parseconfig(newui, options[b'config'])
 
     # stolen from tortoisehg.util.copydynamicconfig()
@@ -354,7 +360,7 @@ class chgcmdserver(commandserver.server):
     def __init__(
         self, ui, repo, fin, fout, sock, prereposetups, hashstate, baseaddress
     ):
-        super(chgcmdserver, self).__init__(
+        super().__init__(
             _newchgui(ui, channeledsystem(fin, fout, b'S'), self.attachio),
             repo,
             fin,
@@ -371,7 +377,7 @@ class chgcmdserver(commandserver.server):
             self.capabilities[b'validate'] = chgcmdserver.validate
 
     def cleanup(self):
-        super(chgcmdserver, self).cleanup()
+        super().cleanup()
         # dispatch._runcatch() does not flush outputs if exception is not
         # handled by dispatch._dispatch()
         self.ui.flush()
@@ -567,7 +573,7 @@ class chgcmdserver(commandserver.server):
         globaloldios = self._oldios
         self._oldios = []
         try:
-            return super(chgcmdserver, self).runcommand()
+            return super().runcommand()
         finally:
             self._restoreio()
             self._oldios = globaloldios
@@ -628,14 +634,16 @@ class chgunixservicehandler:
 
     pollinterval = 1  # [sec]
 
+    _hashstate: Optional[hashstate]
+    _baseaddress: Optional[bytes]
+    _realaddress: Optional[bytes]
+
     def __init__(self, ui):
         self.ui = ui
 
-        # TODO: use PEP 526 syntax (`_hashstate: hashstate` at the class level)
-        #  when 3.5 support is dropped.
-        self._hashstate = None  # type: hashstate
-        self._baseaddress = None  # type: bytes
-        self._realaddress = None  # type: bytes
+        self._hashstate = None
+        self._baseaddress = None
+        self._realaddress = None
 
         self._idletimeout = ui.configint(b'chgserver', b'idletimeout')
         self._lastactive = time.time()
