@@ -5,16 +5,25 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import re
 import struct
+import typing
 import zlib
 
-from .i18n import _
-from .pycompat import (
-    getattr,
-    setattr,
+from typing import (
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+    cast,
 )
+
+from .i18n import _
 from . import (
     diffhelper,
     encoding,
@@ -23,10 +32,14 @@ from . import (
     pycompat,
     util,
 )
+from .interfaces import (
+    modules as intmod,
+)
+
 from .utils import dateutil
 
-bdiff = policy.importmod('bdiff')
-mpatch = policy.importmod('mpatch')
+bdiff: intmod.BDiff = policy.importmod('bdiff')
+mpatch: intmod.MPatch = policy.importmod('mpatch')
 
 blocks = bdiff.blocks
 fixws = bdiff.fixws
@@ -35,6 +48,21 @@ patchedsize = mpatch.patchedsize
 textdiff = bdiff.bdiff
 splitnewlines = bdiff.splitnewlines
 
+if typing.TYPE_CHECKING:
+    HunkLines = List[bytes]
+    """Lines of a hunk- a header, followed by line additions and deletions."""
+
+    HunkRange = Tuple[int, int, int, int]
+    """HunkRange represents the range information of a hunk.
+
+    The tuple (s1, l1, s2, l2) forms the header '@@ -s1,l1 +s2,l2 @@'."""
+
+    Range = Tuple[int, int]
+    """A (lowerbound, upperbound) range tuple."""
+
+    TypedBlock = Tuple[intmod.BDiffBlock, bytes]
+    """A bdiff block with its type."""
+
 
 # TODO: this looks like it could be an attrs, which might help pytype
 class diffopts:
@@ -78,7 +106,7 @@ class diffopts:
             v = opts.get(k)
             if v is None:
                 v = self.defaults[k]
-            setattr(self, k, v)
+            setattr(self, pycompat.sysstr(k), v)
 
         try:
             self.context = int(self.context)
@@ -89,14 +117,15 @@ class diffopts:
             )
 
     def copy(self, **kwargs):
-        opts = {k: getattr(self, k) for k in self.defaults}
+        opts = {k: getattr(self, pycompat.sysstr(k)) for k in self.defaults}
         opts = pycompat.strkwargs(opts)
         opts.update(kwargs)
         return diffopts(**opts)
 
     def __bytes__(self):
         return b", ".join(
-            b"%s: %r" % (k, getattr(self, k)) for k in self.defaults
+            b"%s: %r" % (k, getattr(self, pycompat.sysstr(k)))
+            for k in self.defaults
         )
 
     __str__ = encoding.strmethod(__bytes__)
@@ -105,11 +134,11 @@ class diffopts:
 defaultopts = diffopts()
 
 
-def wsclean(opts, text, blank=True):
+def wsclean(opts: diffopts, text: bytes, blank: bool = True) -> bytes:
     if opts.ignorews:
-        text = bdiff.fixws(text, 1)
+        text = bdiff.fixws(text, True)
     elif opts.ignorewsamount:
-        text = bdiff.fixws(text, 0)
+        text = bdiff.fixws(text, False)
     if blank and opts.ignoreblanklines:
         text = re.sub(b'\n+', b'\n', text).strip(b'\n')
     if opts.ignorewseol:
@@ -117,7 +146,13 @@ def wsclean(opts, text, blank=True):
     return text
 
 
-def splitblock(base1, lines1, base2, lines2, opts):
+def splitblock(
+    base1: int,
+    lines1: Iterable[bytes],
+    base2: int,
+    lines2: Iterable[bytes],
+    opts: diffopts,
+) -> Iterable[TypedBlock]:
     # The input lines matches except for interwoven blank lines. We
     # transform it into a sequence of matching blocks and blank blocks.
     lines1 = [(wsclean(opts, l) and 1 or 0) for l in lines1]
@@ -138,12 +173,12 @@ def splitblock(base1, lines1, base2, lines2, opts):
             while i1 < e1 and lines1[i1] == 1 and lines2[i2] == 1:
                 i1 += 1
                 i2 += 1
-        yield [base1 + s1, base1 + i1, base2 + s2, base2 + i2], btype
+        yield (base1 + s1, base1 + i1, base2 + s2, base2 + i2), btype
         s1 = i1
         s2 = i2
 
 
-def hunkinrange(hunk, linerange):
+def hunkinrange(hunk: Tuple[int, int], linerange: Range) -> bool:
     """Return True if `hunk` defined as (start, length) is in `linerange`
     defined as (lowerbound, upperbound).
 
@@ -169,7 +204,9 @@ def hunkinrange(hunk, linerange):
     return lowerbound < start + length and start < upperbound
 
 
-def blocksinrange(blocks, rangeb):
+def blocksinrange(
+    blocks: Iterable[TypedBlock], rangeb: Range
+) -> Tuple[List[TypedBlock], Range]:
     """filter `blocks` like (a1, a2, b1, b2) from items outside line range
     `rangeb` from ``(b1, b2)`` point of view.
 
@@ -209,18 +246,24 @@ def blocksinrange(blocks, rangeb):
     return filteredblocks, (lba, uba)
 
 
-def chooseblocksfunc(opts=None):
+def chooseblocksfunc(opts: Optional[diffopts] = None) -> intmod.BDiffBlocksFnc:
     if (
         opts is None
         or not opts.xdiff
-        or not util.safehasattr(bdiff, b'xdiffblocks')
+        or not getattr(bdiff, 'xdiffblocks', None)
     ):
         return bdiff.blocks
     else:
         return bdiff.xdiffblocks
 
 
-def allblocks(text1, text2, opts=None, lines1=None, lines2=None):
+def allblocks(
+    text1: bytes,
+    text2: bytes,
+    opts: Optional[diffopts] = None,
+    lines1: Optional[Sequence[bytes]] = None,
+    lines2: Optional[Sequence[bytes]] = None,
+) -> Iterable[TypedBlock]:
     """Return (block, type) tuples, where block is an mdiff.blocks
     line entry. type is '=' for blocks matching exactly one another
     (bdiff blocks), '!' for non-matching blocks and '~' for blocks
@@ -242,8 +285,8 @@ def allblocks(text1, text2, opts=None, lines1=None, lines2=None):
         if i > 0:
             s = diff[i - 1]
         else:
-            s = [0, 0, 0, 0]
-        s = [s[1], s1[0], s[3], s1[2]]
+            s = (0, 0, 0, 0)
+        s = (s[1], s1[0], s[3], s1[2])
 
         # bdiff sometimes gives huge matches past eof, this check eats them,
         # and deals with the special first match case described above
@@ -262,7 +305,16 @@ def allblocks(text1, text2, opts=None, lines1=None, lines2=None):
         yield s1, b'='
 
 
-def unidiff(a, ad, b, bd, fn1, fn2, binary, opts=defaultopts):
+def unidiff(
+    a: bytes,
+    ad: bytes,
+    b: bytes,
+    bd: bytes,
+    fn1: bytes,
+    fn2: bytes,
+    binary: bool,
+    opts: diffopts = defaultopts,
+) -> Tuple[List[bytes], Iterable[Tuple[Optional[HunkRange], HunkLines]]]:
     """Return a unified diff as a (headers, hunks) tuple.
 
     If the diff is not null, `headers` is a list with unified diff header
@@ -273,7 +325,7 @@ def unidiff(a, ad, b, bd, fn1, fn2, binary, opts=defaultopts):
     Set binary=True if either a or b should be taken as a binary file.
     """
 
-    def datetag(date, fn=None):
+    def datetag(date: bytes, fn: Optional[bytes] = None):
         if not opts.git and not opts.nodates:
             return b'\t%s' % date
         if fn and b' ' in fn:
@@ -342,10 +394,16 @@ def unidiff(a, ad, b, bd, fn1, fn2, binary, opts=defaultopts):
             b"+++ %s%s%s" % (bprefix, fn2, datetag(bd, fn2)),
         ]
 
-    return headerlines, hunks
+    # The possible bool is consumed from the iterator above in the `next()`
+    # call.
+    return headerlines, cast(
+        "Iterable[Tuple[Optional[HunkRange], HunkLines]]", hunks
+    )
 
 
-def _unidiff(t1, t2, opts=defaultopts):
+def _unidiff(
+    t1: bytes, t2: bytes, opts: diffopts = defaultopts
+) -> Iterator[Union[bool, Tuple[HunkRange, HunkLines]]]:
     """Yield hunks of a headerless unified diff from t1 and t2 texts.
 
     Each hunk consists of a (hunkrange, hunklines) tuple where `hunkrange` is a
@@ -373,7 +431,9 @@ def _unidiff(t1, t2, opts=defaultopts):
 
     lastfunc = [0, b'']
 
-    def yieldhunk(hunk):
+    def yieldhunk(
+        hunk: Tuple[int, int, int, int, List[bytes]]
+    ) -> Iterable[Tuple[HunkRange, HunkLines]]:
         (astart, a2, bstart, b2, delta) = hunk
         aend = contextend(a2, len(l1))
         alen = aend - astart
@@ -469,16 +529,14 @@ def _unidiff(t1, t2, opts=defaultopts):
                 if not has_hunks:
                     has_hunks = True
                     yield True
-                for x in yieldhunk(hunk):
-                    yield x
+                yield from yieldhunk(hunk)
         if prev:
             # we've joined the previous hunk, record the new ending points.
-            hunk[1] = a2
-            hunk[3] = b2
+            hunk = (hunk[0], a2, hunk[2], b2, hunk[4])
             delta = hunk[4]
         else:
             # create a new hunk
-            hunk = [astart, a2, bstart, b2, delta]
+            hunk = (astart, a2, bstart, b2, delta)
 
         delta[len(delta) :] = [b' ' + x for x in l1[astart:a1]]
         delta[len(delta) :] = [b'-' + x for x in old]
@@ -488,13 +546,12 @@ def _unidiff(t1, t2, opts=defaultopts):
         if not has_hunks:
             has_hunks = True
             yield True
-        for x in yieldhunk(hunk):
-            yield x
+        yield from yieldhunk(hunk)
     elif not has_hunks:
         yield False
 
 
-def b85diff(to, tn):
+def b85diff(to: Optional[bytes], tn: Optional[bytes]) -> bytes:
     '''print base85-encoded binary diff'''
 
     def fmtline(line):
@@ -531,7 +588,7 @@ def b85diff(to, tn):
     return b''.join(ret)
 
 
-def patchtext(bin):
+def patchtext(bin: bytes) -> bytes:
     pos = 0
     t = []
     while pos < len(bin):
@@ -550,13 +607,13 @@ def patch(a, bin):
 
 
 # similar to difflib.SequenceMatcher.get_matching_blocks
-def get_matching_blocks(a, b):
+def get_matching_blocks(a: bytes, b: bytes) -> List[Tuple[int, int, int]]:
     return [(d[0], d[2], d[1] - d[0]) for d in bdiff.blocks(a, b)]
 
 
-def trivialdiffheader(length):
+def trivialdiffheader(length: int) -> bytes:
     return struct.pack(b">lll", 0, 0, length) if length else b''
 
 
-def replacediffheader(oldlen, newlen):
+def replacediffheader(oldlen: int, newlen: int) -> bytes:
     return struct.pack(b">lll", 0, oldlen, newlen)
