@@ -13,6 +13,7 @@ This contains helper routines that are independent of the SCM core and
 hide platform-specific details from the core.
 """
 
+from __future__ import annotations
 
 import abc
 import collections
@@ -25,28 +26,39 @@ import itertools
 import locale
 import mmap
 import os
-import pickle  # provides util.pickle symbol
+
+# provides util.pickle symbol
+import pickle  # noqa: F401 (ignore imported but not used)
 import re as remod
 import shutil
 import stat
 import sys
 import time
 import traceback
+import typing
 import warnings
 
 from typing import (
+    Any,
+    BinaryIO,
+    Callable,
     Iterable,
     Iterator,
     List,
     Optional,
     Tuple,
+    Type,
+    TypeVar,
 )
 
 from .node import hex
 from .thirdparty import attr
-from .pycompat import (
-    open,
-)
+
+# Force pytype to use the non-vendored package
+if typing.TYPE_CHECKING:
+    # noinspection PyPackageRequirements
+    import attr
+
 from hgdemandimport import tracing
 from . import (
     encoding,
@@ -54,8 +66,13 @@ from . import (
     i18n,
     policy,
     pycompat,
+    typelib,
     urllibcompat,
 )
+from .interfaces import (
+    misc as int_misc,
+    modules as intmod,
+)
 from .utils import (
     compression,
     hashutil,
@@ -72,8 +89,14 @@ assert [
     Tuple,
 ]
 
+if typing.TYPE_CHECKING:
+    from typing_extensions import (
+        Self,
+    )
 
-base85 = policy.importmod('base85')
+    _Tcow = TypeVar('_Tcow', bound="cow")
+
+base85: intmod.Base85 = policy.importmod('base85')
 osutil = policy.importmod('osutil')
 
 b85decode = base85.b85decode
@@ -147,6 +170,10 @@ unlink = platform.unlink
 username = platform.username
 
 
+if typing.TYPE_CHECKING:
+    _Tfilestat = TypeVar('_Tfilestat', bound='filestat')
+
+
 def setumask(val: int) -> None:
     '''updates the umask. used by chg server'''
     if pycompat.iswindows:
@@ -322,8 +349,10 @@ except NameError:
 
     def buffer(sliceable, offset=0, length=None):
         if length is not None:
-            return memoryview(sliceable)[offset : offset + length]
-        return memoryview(sliceable)[offset:]
+            view = memoryview(sliceable)[offset : offset + length]
+        else:
+            view = memoryview(sliceable)[offset:]
+        return view.toreadonly()
 
 
 _chunksize = 4096
@@ -350,7 +379,7 @@ class bufferedinputpipe:
         if isinstance(fh, fileobjectproxy):
             cls = observedbufferedinputpipe
 
-        return super(bufferedinputpipe, cls).__new__(cls)
+        return super().__new__(cls)
 
     def __init__(self, input):
         self._input = input
@@ -439,7 +468,25 @@ class bufferedinputpipe:
         return data
 
 
-def mmapread(fp, size=None):
+def has_mmap_populate():
+    return hasattr(osutil, "background_mmap_populate") or hasattr(
+        mmap, 'MAP_POPULATE'
+    )
+
+
+def mmapread(fp, size=None, pre_populate=True):
+    """Read a file content using mmap
+
+    The responsibility of checking the file system is mmap safe is the
+    responsibility of the caller (see `vfs.is_mmap_safe`).
+
+    In some case, a normal string might be returned.
+
+    If `pre_populate` is True (the default), the mmapped data will be
+    pre-populated in memory if the system support this option, this slow down
+    the initial mmapping but avoid potentially crippling page fault on later
+    access. If this is not the desired behavior, set `pre_populate` to False.
+    """
     if size == 0:
         # size of 0 to mmap.mmap() means "all data"
         # rather than "zero bytes", so special case that.
@@ -447,8 +494,24 @@ def mmapread(fp, size=None):
     elif size is None:
         size = 0
     fd = getattr(fp, 'fileno', lambda: fp)()
+
+    if pycompat.iswindows:
+        _mmap = lambda fd, size: mmap.mmap(fd, size, access=mmap.ACCESS_READ)
+    else:
+        flags = mmap.MAP_PRIVATE
+        bg_populate = hasattr(osutil, "background_mmap_populate")
+
+        if pre_populate and not bg_populate:
+            flags |= getattr(mmap, 'MAP_POPULATE', 0)
+
+        def _mmap(fd, size) -> mmap.mmap:
+            m = mmap.mmap(fd, size, flags=flags, prot=mmap.PROT_READ)
+            if pre_populate and bg_populate:
+                osutil.background_mmap_populate(m)
+            return m
+
     try:
-        return mmap.mmap(fd, size, access=mmap.ACCESS_READ)
+        return _mmap(fd, size)
     except ValueError:
         # Empty files cannot be mmapped, but mmapread should still work.  Check
         # if the file is empty, and if so, return an empty buffer.
@@ -457,6 +520,16 @@ def mmapread(fp, size=None):
         raise
 
 
+class uncacheable_cachestat:
+    stat: Optional[os.stat_result]
+
+    def __init__(self) -> None:
+        self.stat = None
+
+    def cacheable(self) -> bool:
+        return False
+
+
 class fileobjectproxy:
     """A proxy around file objects that tells a watcher when events occur.
 
@@ -648,7 +721,7 @@ class observedbufferedinputpipe(bufferedinputpipe):
     """
 
     def _fillbuffer(self, size=_chunksize):
-        res = super(observedbufferedinputpipe, self)._fillbuffer(size=size)
+        res = super()._fillbuffer(size=size)
 
         fn = getattr(self._input._observer, 'osread', None)
         if fn:
@@ -659,7 +732,7 @@ class observedbufferedinputpipe(bufferedinputpipe):
     # We use different observer methods because the operation isn't
     # performed on the actual file object but on us.
     def read(self, size):
-        res = super(observedbufferedinputpipe, self).read(size)
+        res = super().read(size)
 
         fn = getattr(self._input._observer, 'bufferedread', None)
         if fn:
@@ -668,7 +741,7 @@ class observedbufferedinputpipe(bufferedinputpipe):
         return res
 
     def readline(self, *args, **kwargs):
-        res = super(observedbufferedinputpipe, self).readline(*args, **kwargs)
+        res = super().readline(*args, **kwargs)
 
         fn = getattr(self._input._observer, 'bufferedreadline', None)
         if fn:
@@ -859,7 +932,7 @@ class fileobjectobserver(baseproxyobserver):
     def __init__(
         self, fh, name, reads=True, writes=True, logdata=False, logdataapis=True
     ):
-        super(fileobjectobserver, self).__init__(fh, name, logdata, logdataapis)
+        super().__init__(fh, name, logdata, logdataapis)
         self.reads = reads
         self.writes = writes
 
@@ -982,7 +1055,7 @@ class socketobserver(baseproxyobserver):
         logdata=False,
         logdataapis=True,
     ):
-        super(socketobserver, self).__init__(fh, name, logdata, logdataapis)
+        super().__init__(fh, name, logdata, logdataapis)
         self.reads = reads
         self.writes = writes
         self.states = states
@@ -1131,9 +1204,10 @@ def makeloggingsocket(
 def version():
     """Return version information if available."""
     try:
-        from . import __version__
+        from . import __version__  # pytype: disable=import-error
 
-        return __version__.version
+        # setuptools-scm uses py3 str
+        return __version__.version.encode()
     except ImportError:
         return b'unknown'
 
@@ -1262,7 +1336,9 @@ class cow:
     Call preparewrite before doing any writes.
     """
 
-    def preparewrite(self):
+    _copied: int  # doesn't exist until first preparewrite()
+
+    def preparewrite(self: _Tcow) -> _Tcow:
         """call this before writes, return self or a copied new object"""
         if getattr(self, '_copied', 0):
             self._copied -= 1
@@ -1270,7 +1346,7 @@ class cow:
             return self.__class__(self)  # pytype: disable=wrong-arg-count
         return self
 
-    def copy(self):
+    def copy(self) -> Self:
         """always do a cheap copy"""
         self._copied = getattr(self, '_copied', 0) + 1
         return self
@@ -1294,7 +1370,7 @@ class sortdict(collections.OrderedDict):
     def __setitem__(self, key, value):
         if key in self:
             del self[key]
-        super(sortdict, self).__setitem__(key, value)
+        super().__setitem__(key, value)
 
     if pycompat.ispypy:
         # __setitem__() isn't called as of PyPy 5.8.0
@@ -1307,7 +1383,7 @@ class sortdict(collections.OrderedDict):
                 self[k] = f[k]
 
     def insert(self, position, key, value):
-        for (i, (k, v)) in enumerate(list(self.items())):
+        for i, (k, v) in enumerate(list(self.items())):
             if i == position:
                 self[key] = value
             if i >= position:
@@ -1349,26 +1425,24 @@ class cowsortdict(cow, sortdict):
     """
 
 
-class transactional:  # pytype: disable=ignored-metaclass
+class transactional(abc.ABC):
     """Base class for making a transactional type into a context manager."""
 
-    __metaclass__ = abc.ABCMeta
-
     @abc.abstractmethod
-    def close(self):
+    def close(self) -> None:
         """Successfully closes the transaction."""
 
     @abc.abstractmethod
-    def release(self):
+    def release(self) -> None:
         """Marks the end of the transaction.
 
         If the transaction has not been closed, it will be aborted.
         """
 
-    def __enter__(self):
+    def __enter__(self) -> Self:
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
         try:
             if exc_type is None:
                 self.close()
@@ -1812,7 +1886,7 @@ def never(fn):
     return False
 
 
-def nogc(func):
+def nogc(func=None) -> Any:
     """disable garbage collector
 
     Python's garbage collector triggers a GC each time a certain number of
@@ -1825,15 +1899,27 @@ def nogc(func):
     This garbage collector issue have been fixed in 2.7. But it still affect
     CPython's performance.
     """
+    if func is None:
+        return _nogc_context()
+    else:
+        return _nogc_decorator(func)
 
+
+@contextlib.contextmanager
+def _nogc_context():
+    gcenabled = gc.isenabled()
+    gc.disable()
+    try:
+        yield
+    finally:
+        if gcenabled:
+            gc.enable()
+
+
+def _nogc_decorator(func):
     def wrapper(*args, **kwargs):
-        gcenabled = gc.isenabled()
-        gc.disable()
-        try:
+        with _nogc_context():
             return func(*args, **kwargs)
-        finally:
-            if gcenabled:
-                gc.enable()
 
     return wrapper
 
@@ -1947,7 +2033,7 @@ def copyfile(
                 m = "the `nb_bytes` argument is incompatible with `hardlink`"
                 raise error.ProgrammingError(m)
             return
-        except (IOError, OSError) as exc:
+        except OSError as exc:
             if exc.errno != errno.EEXIST and no_hardlink_cb is not None:
                 no_hardlink_cb()
             # fall back to normal copy
@@ -2013,7 +2099,7 @@ def copyfiles(src, dst, hardlink=None, progress=None):
         if hardlink:
             try:
                 oslink(src, dst)
-            except (IOError, OSError) as exc:
+            except OSError as exc:
                 if exc.errno != errno.EEXIST:
                     hardlink = False
                 # XXX maybe try to relink if the file exist ?
@@ -2128,7 +2214,7 @@ else:
         timer = time.time
 
 
-def makelock(info, pathname):
+def makelock(info: bytes, pathname: bytes) -> None:
     """Create a lock file atomically if possible
 
     This may leave a stale lock file if symlink isn't supported and signal
@@ -2144,8 +2230,10 @@ def makelock(info, pathname):
 
     flags = os.O_CREAT | os.O_WRONLY | os.O_EXCL | getattr(os, 'O_BINARY', 0)
     ld = os.open(pathname, flags)
-    os.write(ld, info)
-    os.close(ld)
+    try:
+        os.write(ld, info)
+    finally:
+        os.close(ld)
 
 
 def readlock(pathname: bytes) -> bytes:
@@ -2384,7 +2472,12 @@ def splitpath(path: bytes) -> List[bytes]:
     return path.split(pycompat.ossep)
 
 
-def mktempcopy(name, emptyok=False, createmode=None, enforcewritable=False):
+def mktempcopy(
+    name: bytes,
+    emptyok: bool = False,
+    createmode: Optional[int] = None,
+    enforcewritable: bool = False,
+) -> bytes:
     """Create a temporary file with the same contents from name
 
     The permission bits are copied from the original file.
@@ -2407,7 +2500,7 @@ def mktempcopy(name, emptyok=False, createmode=None, enforcewritable=False):
     try:
         try:
             ifp = posixfile(name, b"rb")
-        except IOError as inst:
+        except OSError as inst:
             if inst.errno == errno.ENOENT:
                 return temp
             if not getattr(inst, 'filename', None):
@@ -2435,11 +2528,11 @@ class filestat:
     'exists()' examination on client side of this class.
     """
 
-    def __init__(self, stat):
+    def __init__(self, stat: Optional[os.stat_result]) -> None:
         self.stat = stat
 
     @classmethod
-    def frompath(cls, path):
+    def frompath(cls: Type[_Tfilestat], path: bytes) -> _Tfilestat:
         try:
             stat = os.stat(path)
         except FileNotFoundError:
@@ -2447,13 +2540,13 @@ class filestat:
         return cls(stat)
 
     @classmethod
-    def fromfp(cls, fp):
+    def fromfp(cls: Type[_Tfilestat], fp: BinaryIO) -> _Tfilestat:
         stat = os.fstat(fp.fileno())
         return cls(stat)
 
     __hash__ = object.__hash__
 
-    def __eq__(self, old):
+    def __eq__(self, old) -> bool:
         try:
             # if ambiguity between stat of new and old file is
             # avoided, comparison of size, ctime and mtime is enough
@@ -2470,7 +2563,7 @@ class filestat:
         except AttributeError:
             return False
 
-    def isambig(self, old):
+    def isambig(self, old: _Tfilestat) -> bool:
         """Examine whether new (= self) stat is ambiguous against old one
 
         "S[N]" below means stat of a file at N-th change:
@@ -2505,7 +2598,7 @@ class filestat:
         except AttributeError:
             return False
 
-    def avoidambig(self, path, old):
+    def avoidambig(self, path: bytes, old: _Tfilestat) -> bool:
         """Change file stat of specified path to avoid ambiguity
 
         'old' should be previous filestat of 'path'.
@@ -2525,7 +2618,7 @@ class filestat:
             return False
         return True
 
-    def __ne__(self, other):
+    def __ne__(self, other) -> bool:
         return not self == other
 
 
@@ -2668,17 +2761,17 @@ def makedirs(
 
 
 def readfile(path: bytes) -> bytes:
-    with open(path, b'rb') as fp:
+    with open(path, 'rb') as fp:
         return fp.read()
 
 
 def writefile(path: bytes, text: bytes) -> None:
-    with open(path, b'wb') as fp:
+    with open(path, 'wb') as fp:
         fp.write(text)
 
 
 def appendfile(path: bytes, text: bytes) -> None:
-    with open(path, b'ab') as fp:
+    with open(path, 'ab') as fp:
         fp.write(text)
 
 
@@ -2691,10 +2784,10 @@ class chunkbuffer:
 
         def splitbig(chunks):
             for chunk in chunks:
-                if len(chunk) > 2 ** 20:
+                if len(chunk) > 2**20:
                     pos = 0
                     while pos < len(chunk):
-                        end = pos + 2 ** 18
+                        end = pos + 2**18
                         yield chunk[pos:end]
                         pos = end
                 else:
@@ -2703,6 +2796,25 @@ class chunkbuffer:
         self.iter = splitbig(in_iter)
         self._queue = collections.deque()
         self._chunkoffset = 0
+        self._absolute_offset = 0
+
+    def __iter__(self):
+        while self._queue:
+            chunk = self._queue.popleft()
+            if self._chunkoffset:
+                d = chunk[self._chunkoffset :]
+            else:
+                d = chunk
+            self._absolute_offset += len(d)
+            yield d
+            self._chunkoffset = 0
+        for d in self.iter:
+            self._absolute_offset += len(d)
+            yield d
+
+    def tell(self) -> int:
+        """tell how much data we have read so far"""
+        return self._absolute_offset
 
     def read(self, l=None):
         """Read L bytes of data from the iterator of chunks of data.
@@ -2710,7 +2822,9 @@ class chunkbuffer:
 
         If size parameter is omitted, read everything"""
         if l is None:
-            return b''.join(self.iter)
+            d = b''.join(self.iter)
+            self._absolute_offset += len(d)
+            return d
 
         left = l
         buf = []
@@ -2718,7 +2832,7 @@ class chunkbuffer:
         while left > 0:
             # refill the queue
             if not queue:
-                target = 2 ** 18
+                target = 2**18
                 for chunk in self.iter:
                     queue.append(chunk)
                     target -= len(chunk)
@@ -2762,10 +2876,15 @@ class chunkbuffer:
                 self._chunkoffset += left
                 left -= chunkremaining
 
-        return b''.join(buf)
+        d = b''.join(buf)
+        self._absolute_offset += len(d)
+        return d
 
 
-def filechunkiter(f, size=131072, limit=None):
+DEFAULT_FILE_CHUNK = 128 * (2**10)
+
+
+def filechunkiter(f, size=DEFAULT_FILE_CHUNK, limit=None):
     """Create a generator that produces the data in the file size
     (default 131072) bytes at a time, up to optional limit (default is
     to read all data).  Chunks may be less than size bytes if the
@@ -2874,20 +2993,20 @@ bytecount = unitcountfn(
 )
 
 
-class transformingwriter:
+class transformingwriter(typelib.BinaryIO_Proxy):
     """Writable file wrapper to transform data by function"""
 
-    def __init__(self, fp, encode):
+    def __init__(self, fp: BinaryIO, encode: Callable[[bytes], bytes]) -> None:
         self._fp = fp
         self._encode = encode
 
-    def close(self):
+    def close(self) -> None:
         self._fp.close()
 
-    def flush(self):
+    def flush(self) -> None:
         self._fp.flush()
 
-    def write(self, data):
+    def write(self, data: bytes) -> int:
         return self._fp.write(self._encode(data))
 
 
@@ -2905,7 +3024,7 @@ def tocrlf(s: bytes) -> bytes:
     return _eolre.sub(b'\r\n', s)
 
 
-def _crlfwriter(fp):
+def _crlfwriter(fp: typelib.BinaryIO_Proxy) -> typelib.BinaryIO_Proxy:
     return transformingwriter(fp, tocrlf)
 
 
@@ -2918,6 +3037,21 @@ else:
     fromnativeeol = pycompat.identity
     nativeeolwriter = pycompat.identity
 
+if typing.TYPE_CHECKING:
+    # Replace the various overloads that come along with aliasing other methods
+    # with the narrow definition that we care about in the type checking phase
+    # only.  This ensures that both Windows and POSIX see only the definition
+    # that is actually available.
+
+    def tonativeeol(s: bytes) -> bytes:
+        raise NotImplementedError
+
+    def fromnativeeol(s: bytes) -> bytes:
+        raise NotImplementedError
+
+    def nativeeolwriter(fp: typelib.BinaryIO_Proxy) -> typelib.BinaryIO_Proxy:
+        raise NotImplementedError
+
 
 # TODO delete since workaround variant for Python 2 no longer needed.
 def iterfile(fp):
@@ -2926,8 +3060,7 @@ def iterfile(fp):
 
 def iterlines(iterator: Iterable[bytes]) -> Iterator[bytes]:
     for chunk in iterator:
-        for line in chunk.splitlines():
-            yield line
+        yield from chunk.splitlines()
 
 
 def expandpath(path: bytes) -> bytes:
@@ -3048,12 +3181,12 @@ def timed(func):
 
 
 _sizeunits = (
-    (b'm', 2 ** 20),
-    (b'k', 2 ** 10),
-    (b'g', 2 ** 30),
-    (b'kb', 2 ** 10),
-    (b'mb', 2 ** 20),
-    (b'gb', 2 ** 30),
+    (b'm', 2**20),
+    (b'k', 2**10),
+    (b'g', 2**30),
+    (b'kb', 2**10),
+    (b'mb', 2**20),
+    (b'gb', 2**30),
     (b'b', 1),
 )
 
@@ -3078,7 +3211,7 @@ def sizetoint(s: bytes) -> int:
         raise error.ParseError(_(b"couldn't parse size: %s") % s)
 
 
-class hooks:
+class hooks(int_misc.IHooks):
     """A collection of hook functions that can be used to extend a
     function's behavior. Hooks are called in lexicographic order,
     based on the names of their sources."""
@@ -3086,10 +3219,10 @@ class hooks:
     def __init__(self):
         self._hooks = []
 
-    def add(self, source, hook):
+    def add(self, source: bytes, hook: Callable) -> None:
         self._hooks.append((source, hook))
 
-    def __call__(self, *args):
+    def __call__(self, *args) -> List:
         self._hooks.sort(key=lambda x: x[0])
         results = []
         for source, hook in self._hooks:
@@ -3287,6 +3420,7 @@ def _estimatememory() -> Optional[int]:
     """
     if pycompat.sysplatform.startswith(b'win'):
         # On Windows, use the GlobalMemoryStatusEx kernel function directly.
+        # noinspection PyPep8Naming
         from ctypes import c_long as DWORD, c_ulonglong as DWORDLONG
         from ctypes.wintypes import (  # pytype: disable=import-error
             Structure,
